import base64
import os
from xml.dom import minicompat, minidom

import arrow


class OpfParser():
    def __init__(self, opf_doc: minidom.Document, opf_path: str):
        self.opf_doc = opf_doc
        self.opf_path = opf_path

    def __tag_filter(self, tag: str, attr: str, value: str) -> minidom.Element:
        for element in self.opf_doc.getElementsByTagName(tag):
            element: minidom.Element
            if attr in element.attributes.keys() and element.attributes[attr].value == value:
                return element

    def __dc_filter(self, dc_type: str) -> minicompat.NodeList[minidom.Element]:
        return self.opf_doc.getElementsByTagName(f'dc:{dc_type}')

    def version(self) -> str:
        """parser version from opf file

        Returns:
            str: version
        """
        return self.opf_doc.getElementsByTagName('package')[0].attributes['version'].value

    def title(self) -> str:
        """parser title from opf file

        Returns:
            str: title
        """
        elements = self.__dc_filter('title')
        if any(elements) == False:
            return ''
        return elements[0].firstChild.data

    def creator(self) -> str:
        """parser creator from opf file

        Returns:
            str: creator
        """
        element = self.__dc_filter('creator')
        if any(element) == False:
            return ''
        return element[0].firstChild.data

    def date(self) -> str:
        """parser public date from opf file

        Returns:
            str: public date
        """
        elements = self.__dc_filter('date')
        if any(elements) == False:
            return ''
        data = arrow.get(elements[0].firstChild.data).format('YYYY-MM-DD')
        return data

    def description(self) -> str:
        """parser description from opf file

        Returns:
            str: description
        """
        elements = self.__dc_filter('description')
        if any(elements) == False:
            return ''
        return elements[0].firstChild.data

    def publisher(self) -> str:
        """parser publisher from opf file

        Returns:
            str: publisher
        """
        elements = self.__dc_filter('publisher')
        if any(elements) == False:
            return ''
        return elements[0].firstChild.data

    def identifier(self) -> str:
        """parser identifier from opf file

        Returns:
            str: identifier
        """
        elements = self.__dc_filter('identifier')
        if any(elements) == False:
            return ''
        return elements[0].firstChild.data

    def cover(self) -> tuple[str]:
        """parser cover from opf file

        Returns:
            tuple[str]: cover base64 str and cover type
        """
        meta_element = self.__tag_filter('meta', 'name', 'cover')
        if meta_element == None or 'content' not in meta_element.attributes.keys():
            return ('', '')
        item_id = meta_element.attributes['content'].value
        item_element = self.__tag_filter('item', 'id', item_id)
        if item_element == None or 'href' not in item_element.attributes.keys():
            return ('', '')
        cover_path = os.path.join(os.path.dirname(
            self.opf_path), item_element.attributes['href'].value)
        cover_type = item_element.attributes['media-type'].value
        with open(cover_path, 'rb') as f:
            cover_data = base64.b64encode(f.read())
            return (cover_data.decode('utf-8'), cover_type)



import os
import tempfile
import zipfile as zf
from xml.dom import minidom

from .metadata import Metadata
from .opf_parser import OpfParser


class epub(OpfParser):
    def __init__(self, epub_filepath: str):
        self.epub_filepath = epub_filepath

    def __unzip(self, tempdir: str) -> None:
        """unzip epub file to tempdir

        Args:
            tempdir (str): tempdir
        """
        zipfile = zf.ZipFile(self.epub_filepath)
        for names in zipfile.namelist():
            zipfile.extract(names, tempdir)

    def __opf_path(self, container_xml_path: str) -> str:
        """get .opf path from container.xml

        Args:
            container_xml_path (str): container.xml path

        Returns:
            str: .opf path
        """
        container_doc: minidom.Document = minidom.parse(
            container_xml_path)
        opf_path = container_doc.getElementsByTagName(
            'rootfile')[0].attributes['full-path'].value
        return opf_path

    @property
    def metadata(self) -> Metadata:
        """get metadata

        Returns:
            Metadata: metadata object
        """
        with tempfile.TemporaryDirectory(prefix='epub_') as tempdir:
            self.__unzip(tempdir)
            container_xml_path = os.path.join(
                tempdir, 'META-INF', 'container.xml')
            opf_filepath = os.path.join(
                tempdir, self.__opf_path(container_xml_path))
            opf_doc: minidom.Document = minidom.parse(opf_filepath)
            super().__init__(opf_doc, opf_filepath)
            cover_base64, cover_type = self.cover()
            _metadata = {
                'version': self.version(),
                'title': self.title(),
                'creator': self.creator(),
                'date': self.date(),
                'cover': cover_base64,
                'cover_type': cover_type,
                'description': self.description(),
                'publisher': self.publisher(),
                'identifier': self.identifier(),
            }
            return Metadata(_metadata)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['epub_metadata']

package_data = \
{'': ['*']}

install_requires = \
['arrow>=1.2.3,<2.0.0']

setup_kwargs = {
    'name': 'epub-metadata',
    'version': '1.0.1b0',
    'description': '',
    'long_description': '# epub-metadata\nGet metadata from the Epub file.\n\n# Install\n```bash\npip install epub-metadata\n```\n\n# Example\n\n```python\nimport epub_metadata\nepub = epub_metadata.epub(\'tests/Alices Adventures in Wonderland.epub\')\n```\n\nshow all metadata from the Epub file\n\n```python\nprint(epub.metadata)\n# return all metadata from the Epub file\n{\n    \'version\': \'2.0\', \n    \'title\': "Alice\'s Adventures in Wonderland", \n    \'creator\': \'Lewis Carroll\', \n    \'date\': \'1865-07-04\', \n    \'cover\': \'/9j/4AAQSkZJRgABAQE...\', \n    \'cover_type\': \'image/jpeg\', \n    \'description\': \'\', \n    \'publisher\': \'D. Appleton and Co\', \n    \'identifier\': \'eb2934ae-bb1a-4652-bce7-9f78fc5ca496\'\n}\n```\n\nonly show the epub metadata\n```python\nprint(epub.metadata.title)\n# only print the title\nAlice\'s Adventures in Wonderland\n```',
    'author': 'ThanatosDi',
    'author_email': 'yykkold55tw@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
